import CodeFrameError from "code-frame-error";
function parseObjectLookupString(path, { escape = "\\" } = {}) {
  if (escape.length !== 1) {
    throw new Error(`Escape char must have length 1, got ${escape.length}`);
  }
  if (!path?.trim()) {
    return [];
  }
  const ctx = path.split("").reduce(
    (ctx2, char) => {
      if (ctx2.state?.type === "escape") {
        if (char === escape || char === ctx2.state.parent.kind) {
          ctx2.state.parent.value += char;
          ctx2.state = ctx2.state.parent;
          return ctx2;
        } else {
          ctx2.state.parent.value += escape;
          ctx2.state = ctx2.state.parent;
        }
      }
      if (char.match(/\r/) && ctx2.state?.type !== "string") {
        return ctx2;
      }
      ctx2.column += 1;
      if (char.match(/\n/)) {
        ctx2.line += 1;
        ctx2.column = 1;
        if (ctx2.state?.type !== "string") {
          return ctx2;
        }
      }
      if (char.match(/\s/) && ctx2.state?.type !== "string") {
        return ctx2;
      }
      if (ctx2.state?.type === "string") {
        if (char === escape) {
          ctx2.state = { type: "escape", parent: ctx2.state };
          return ctx2;
        }
        if (char === ctx2.state.kind) {
          ctx2.token = ctx2.state.value;
          ctx2.state = ctx2.state.parent;
        } else {
          ctx2.state.value += char;
        }
        return ctx2;
      }
      if (ctx2.state?.type === "dot") {
        ctx2.state = ctx2.state.parent;
        push(ctx2);
      }
      if (ctx2.state?.type === "access expression") {
        if (char.match(/[0-9]/)) {
          ctx2.token = (ctx2.token || "") + char;
          return ctx2;
        }
        if (!ctx2.token && isStringDelimiter(char)) {
          ctx2.state.with = "string";
          ctx2.state = {
            type: "string",
            parent: ctx2.state,
            value: "",
            kind: char
          };
          return ctx2;
        }
        if (ctx2.token && char === "]") {
          if (ctx2.state.with === "number") {
            ctx2.token = parseInt(ctx2.token, 10);
          }
          push(ctx2);
          return ctx2;
        }
        throw new CodeFrameError(`Unexpected ${char}`, {
          rawLines: path,
          location: { start: { column: ctx2.column, line: ctx2.line } }
        });
      }
      if (char === "?") {
        ctx2.state = { type: "optional", parent: ctx2.state };
        return ctx2;
      }
      if (char === ".") {
        ctx2.state = { type: "dot", parent: ctx2.state };
        return ctx2;
      }
      if (char === "[") {
        if (ctx2.token !== void 0) {
          push(ctx2);
        }
        ctx2.state = {
          type: "access expression",
          parent: ctx2.state,
          with: "number"
        };
        return ctx2;
      }
      if (!ctx2.token && char.match(/[a-zA-Z_$]/)) {
        ctx2.token = char;
        return ctx2;
      }
      if (typeof ctx2.token === "string" && char.match(/[\w_$]/)) {
        ctx2.token += char;
        return ctx2;
      }
      throw new CodeFrameError(`Unexpected ${char}`, {
        rawLines: path,
        location: { start: { column: ctx2.column, line: ctx2.line } }
      });
    },
    {
      path: [],
      column: 1,
      line: 1,
      optional: false
    }
  );
  if (ctx.state) {
    throw new CodeFrameError(`Unterminated ${ctx.state.type}`, {
      rawLines: path,
      location: { start: { column: ctx.column, line: ctx.line } }
    });
  }
  push(ctx);
  return ctx.path;
}
function push(ctx) {
  if (ctx.token !== void 0) {
    ctx.path.push({
      key: ctx.token,
      optional: ctx.optional
    });
  }
  ctx.optional = ctx.state?.type === "optional";
  ctx.state = ctx.state?.parent;
  ctx.token = void 0;
}
function isStringDelimiter(char) {
  return ["'", '"', "`"].includes(char);
}
var parseObjectLookupString_default = parseObjectLookupString;
export {
  parseObjectLookupString_default as default,
  parseObjectLookupString
};
